package jasymca;

/*
   Copyright (C) 2006 - Helmut Dersch  der@hs-furtwangen.de
   
   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2, or (at your option)
   any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.  */

/*------------------------------------------------------------*/

import java.util.*;

// Strategy:
// (1) Normalize equations
// (2) Solve for all linear variables; use pivoting
// (3) Solve remaining equations using SOLVE
// (4) Keep multiple solutions
// (5) Return as linear factors (x-solution) (might look weird)

class LambdaALGSYS extends Lambda{
 public int lambda(Stack st) throws ParseException, JasymcaException{
  int narg = getNarg( st );
  if(narg!=2)
   throw new ParseException("algsys requires 2 arguments.");

  Algebraic expr = getAlgebraic(st).rat();
  Algebraic vars = getAlgebraic(st);

  if(!(expr instanceof jasymca.Vektor) || !(vars instanceof jasymca.Vektor)
   || ((Vektor)expr).length() != ((Vektor)vars).length() )
   throw new ParseException("Wrong type of arguments to algsys.");


  // Normalize all equations
  expr = new ExpandUser().f_exakt(expr);
  expr = new TrigExpand().f_exakt( expr );
  expr = new NormExp().f_exakt(expr);
  expr = new CollectExp(expr).f_exakt(expr);
  expr = new SqrtExpand().f_exakt(expr);


  Vector v = new Vector();
  for(int i=0; i<((Vektor)vars).length(); i++){
   Algebraic p = ((Vektor)vars).get(i);
   if(!(p instanceof jasymca.Polynomial))
    throw new ParseException("Wrong type of arguments to algsys.");
   v.addElement(((Polynomial)p).var);
  }

  st.push( solvesys(((Vektor)expr).vector(), v));
  return 0;
 }


 Vektor solvesys( Vector expr, Vector x) throws JasymcaException{
  int nvars = x.size();
  Vector lsg = new Vector(),
       vars = new Vector();
  lsg.addElement(expr);
  vars.addElement(x);
  int n = nvars;
  while(n>0){
   for(int i=0; i<lsg.size(); i++){
    try{
     Vector equ = (Vector)lsg.elementAt(i);
     Vector xv = (Vector)vars.elementAt(i);
     Vektor sol = solve(equ, xv, n);
     lsg.removeElementAt(i);
     vars.removeElementAt(i);
     Variable v = (Variable)xv.elementAt(n-1); // Eliminated variable
     for(int k=0; k<sol.length(); k++){ // Insert solution
      Vector eq = new Vector();
      for(int j=0; j<n-1; j++) // Insert solution into remaining equations
       eq.addElement(((Algebraic)equ.elementAt(j)).value(v, sol.get(k)));
      eq.addElement(sol.get(k)); // Insert solution into current position
      for(int j=n; j<nvars; j++)
       eq.addElement(equ.elementAt(j)); // Copy old solutions					
      lsg.insertElementAt(eq, i);
      vars.insertElementAt(clonev(xv),i);
      i++;
     }
    }catch(JasymcaException je){ // Could not find a solution
     lsg.removeElementAt(i);
     vars.removeElementAt(i);
     i--;
    }
   }
   if(lsg.size()==0)
    throw new JasymcaException("Could not solve equations.");
   n--;
  }
  // Backsubstitution		
  for(int i=0; i<lsg.size(); i++){
   Vector equ = (Vector)lsg.elementAt(i);
   Vector xv = (Vector)vars.elementAt(i);
   for(n=1; n<nvars; n++){
    Algebraic y = (Algebraic)equ.elementAt(n-1);
    Variable v = (Variable)xv.elementAt(n-1);
    for(int k=n; k<nvars; k++){
     Algebraic z = (Algebraic)equ.elementAt(k);
     equ.removeElementAt(k);
     equ.insertElementAt(z.value(v,y), k);
    }
   }
  }
  // Convert solutions to linear factors
  for(int i=0; i<lsg.size(); i++){
   Vector equ = (Vector)lsg.elementAt(i);
   Vector xv = (Vector)vars.elementAt(i);
   for(n=0; n<nvars; n++){
    Variable v = (Variable)xv.elementAt(n);
    Algebraic y = new Polynomial(v).sub((Algebraic)equ.elementAt(n));
    equ.removeElementAt(n);
    equ.insertElementAt(y, n);
   }
  }

  // return as Vektor of Vektors
  Vektor[] r = new Vektor[lsg.size()];
  for(int i=0; i<lsg.size(); i++){
   r[i] = Vektor.create((Vector)lsg.elementAt(i));
  }
  return new Vektor(r);
 }

 Vector clonev(Vector v){
  Vector r = new Vector(v.size());
  for(int i=0; i<v.size(); i++)
   r.addElement(v.elementAt(i));
  return r;
 }


 // Solve one of expr and return solution
 // Use coefficients up to (not including) n
 Vektor solve(Vector expr, Vector x, int n) throws JasymcaException{
  // Find linear equation
  Algebraic equ=null;
  Variable v=null;
  int i,k,iv=0,ke=0;
  for(i=0; i<n && equ==null; i++){ // Loop through Variables
   v = (Variable)x.elementAt(i);
   double norm=-1.;
   for(k=0; k<n; k++){ // Loop through equations
    Algebraic exp = (Algebraic)expr.elementAt(k);
    if(exp instanceof jasymca.Rational)
     exp = ((Rational)exp).nom;
    Algebraic slope = exp.deriv(v);
    if(!slope.equals(Zahl.ZERO) && slope instanceof jasymca.Zahl){ // v is linear
     double nm = slope.norm()/exp.norm();
     if(nm>norm){
      norm=nm;
      equ =exp;
      ke=k;
      iv=i;
     }
    }
   }
  }
  if(equ==null){ // Search any dependend Variable	
   for(i=0; i<n && equ==null; i++){ // Loop through Variables
    v = (Variable)x.elementAt(i);
    for(k=0; k<n; k++){ // Loop through equations
     Algebraic exp = (Algebraic)expr.elementAt(k);
     if(exp instanceof jasymca.Rational)
      exp = ((Rational)exp).nom;
     if(exp.depends(v)){
      equ=exp;
      ke=k;
      iv=i;
      break;
     }
    }
   }
  }
  if(equ==null)
   throw new JasymcaException("Expressions do not depend of Variables.");

  Vektor sol = LambdaSOLVE.solve(equ, v);
  expr.removeElementAt(ke);
  expr.insertElementAt(equ, n-1);
  x.removeElementAt(iv);
  x.insertElementAt(v,n-1);
  return sol;
 }


}


/*
@GROUP
equations 
@SYNTAX
algsys([expressions],[symbolic variables]).
@DOC
Solving coupled systems of equations 
@NOTES
@EXAMPLES
syms xs,a0,a1,a2
a=algsys([2-a0,a1-0,a2*xs^2+a1*xs+a0-3-xs, 2*a2*xs+a1+1],[a2,a1,a0,xs])
printf('%f\n',a)

Answer:
[ [ xs+2/3  a2-3/4  a0-2  a1 ] ]
[ [ -sqrt(6)+(xs+3)  2*sqrt(6)+(a-6) ]  [ sqrt(6)+(xs+3)  -2*sqrt(6)+(a-6) ] ]
[ [ xs+0.55051  a-1.101 ]  [ xs+5.4495  a-10.899 ] ]

@SEE
*/

