package jasymca;

import jasymca.JasymcaException;
import jasymca.Lambda;
import jasymca.Matrix;
import jasymca.ParseException;

import java.util.Stack;



/** LU Decomposition.
<P>
For an m-by-n matrix A with m >= n, the LU decomposition is an m-by-n
unit lower triangular matrix L, an n-by-n upper triangular matrix U,
and a permutation vector piv of length m so that A(piv,:) = L*U.
If m < n, then L is m-by-m and U is m-by-n.
<P>
The LU decompostion with pivoting always exists, even if the matrix is
singular, so the constructor will never fail.  The primary use of the
LU decomposition is in the solution of square systems of simultaneous
linear equations.  This will fail if isNonsingular() returns false.
<p>
usage: [L,U]   = lu (A) <br>
       [L,U,P] = lu (A) <br>
       [L]     = lu (A) <br>
       x       = lu (A,B) as a solution to A*X = B;
*/

class LambdaLU extends Lambda{
 public int lambda(Stack st) throws ParseException, JasymcaException{
  int narg = getNarg( st );
  Matrix m = new Matrix( getAlgebraic( st ) ).copy();
  Matrix B = new Matrix(1,1);
  Matrix P = new Matrix(1,1);
  m.rank_decompose( B, P );
  if(length != 2 && length != 3)
   throw new JasymcaException("Usage: [l,u,p] = LU( Matrix ).");
  if( length>=2 ){
   st.push( B );
   st.push( m );
   if(length==3)
    st.push( P );
  }
  length = 1;
  return 0;
 }
}


/*
@GROUP
matrix
@SYNTAX
   [L,U]   = lu (A) 
   [L,U,P] = lu (A) 
   [L]     = lu (A) 
   x       = lu (A,B) as a solution to A*X = B;
@DOC
Returns the LU decomposition of a matrix
@NOTES
   For an m-by-n matrix A with m &gt;= n, the LU decomposition is an m-by-n
   unit lower triangular matrix L, an n-by-n upper triangular matrix U,
   and a permutation vector piv of length m so that A(piv,:) = L*U.
   If m &lt; n, then L is m-by-m and U is m-by-n.
   
   The LU decompostion with pivoting always exists, even if the matrix is
   singular, so the constructor will never fail.  The primary use of the
   LU decomposition is in the solution of square systems of simultaneous
   linear equations.  This will fail if isNonsingular() returns false.
@EXAMPLES
@SEE
qr
*/
